package rec;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

/**
 * This class implements the item-based collaborative filtering algorithm
 *
 */
public class ItemBasedPredictor extends Predictor{

	private static int neighbourhoodSize;
	private HashMap<Integer, Double> averageRatings;

	//hash map: userID --> hashmap: userID --> similarity
	HashMap<Integer, LinkedHashMap<Integer,Double>> similarities;

	//hash map: userID --> Set of movies that the user has rated
	//note that this map is sorted (==> has always the same iteration order)
	private TreeMap<Integer, HashSet<Integer>> UserRatedMovies;

	/*hash map: userID --> hash map: movieID --> rating
	 * for every user a hash map of movie ratings
	 * note that this map is sorted (==> has always the same iteration order)
	 */
	private TreeMap<Integer, HashMap<Integer, Integer>> UserMovieRatings;

	//hash map: movieID --> Set of users that have rated the movie
	private TreeMap<Integer, HashSet<Integer>> UsersByMovie;

	//needed to sort the neighbour lists by similarity
	private static Comparator<Map.Entry<Integer, Double>> comparator;

	public ItemBasedPredictor(int nSize) {
		neighbourhoodSize = nSize;
		UserRatedMovies = new TreeMap<Integer, HashSet<Integer>>();
		UserMovieRatings = new TreeMap<Integer, HashMap<Integer, Integer>>();
		averageRatings = new HashMap<Integer, Double>();
		similarities = new HashMap<Integer, LinkedHashMap<Integer, Double>>();
		UsersByMovie = new TreeMap<Integer, HashSet<Integer>>();

		comparator = new Comparator<Map.Entry<Integer, Double>>() {
			public int compare( Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2 ) {
				return (o1.getValue()).compareTo( o2.getValue() );
			}
		};
	}


	/**
	 * This method trains the predictor using the data given in the input array 'data'
	 * @param data: an array of triples (user, movie, rating)
	 */
	@Override
	public void train(int[][] data) {
		// TODO: implement this function
	}

	/**
	 * This method computes the predicted rating of user 'userID' for the movie 'movieID'
	 * using the weighted-sum prediction
	 * @param userID: the id of the user (as given in the ml100 data file)
	 * @param movieID: the id of the movie (as given in the ml100 data file)
	 * @return: predicted rating
	 */
	@Override
	public double predict(int userID, int movieID) {
		//TODO: implement this function
		// use the weighed-sum prediction: Eq. 20.6
		double prediction = 0.0; 

		return prediction;
	}

	/**
	 * Computes the similarity between movie1 and movie2 using the adjusted cosine similarity
	 * @param movie1
	 * @param movie2
	 * @return similarity
	 */
	public double computeSimilarity(int movie1, int movie2){

		//TODO: implement this function
		//the formula for the adjusted cosine similarity can be found on page 13 of chapter 20 of the lecture notes
		double sim = 0;

		return sim;
	}

	/**
	 * Populate the hash map UserRatedMovies with the data from the input file 
	 * @param data: the input data file
	 */
	public void initializeUserRatedMovies(int[][] data){
		for (int i = 0; i< data.length; i++){
			//user already in the hash map
			if (UserRatedMovies.containsKey(data[i][0])) {
				HashSet<Integer> movies = UserRatedMovies.get(data[i][0]);
				movies.add(data[i][1]);
			}
			else{
				HashSet<Integer> movies = new HashSet<Integer>();
				movies.add(data[i][1]);
				UserRatedMovies.put(data[i][0], movies);
			}

		}
	}

	/**
	 * Populate the hash set UserMovieRatings with the data from the input file
	 * @param data: the input data file
	 */
	public void initializeUserMovieRatings(int[][] data){
		for (int i = 0; i< data.length; i++){
			if (UserMovieRatings.containsKey(data[i][0])) {
				UserMovieRatings.get(data[i][0]).put(data[i][1], data[i][2]);
			}
			else{
				HashMap<Integer, Integer> ratings = new HashMap<Integer,Integer>();
				ratings.put(data[i][1], data[i][2]);
				UserMovieRatings.put(data[i][0], ratings);
			}
		}
	}

	/**
	 * Populate the hash map MoviesByUser with the data from the input file
	 * @param data: input data file
	 */
	public void initializeMoviesByUsers(int[][] data){
		for (int i = 0; i< data.length; i++){
			if (UsersByMovie.containsKey(data[i][1])){
				UsersByMovie.get(data[i][1]).add(data[i][0]);
			}
			else{
				HashSet<Integer> raters = new HashSet<Integer>();
				raters.add(data[i][0]);
				UsersByMovie.put(data[i][1], raters);
			}
		}
	}

	/**
	 * Compute the average ratings for all users
	 * @param data: input data
	 */
	public void computeUserAverageRatings(int[][] data){

		for (Integer userID : UserRatedMovies.keySet()){
			double rating = 0;
			for(Integer movieID : UserRatedMovies.get(userID)){
				rating += UserMovieRatings.get(userID).get(movieID);
			}
			rating /= (double) UserRatedMovies.get(userID).size();
			averageRatings.put(userID, rating);
		}
	}


	public HashMap<Integer, LinkedHashMap<Integer, Double>> getSimilarities() {
		return similarities;
	}


	public TreeMap<Integer, HashSet<Integer>> getUsersByMovie() {
		return UsersByMovie;
	}



}
