package rec;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

/**
 * This class implements the user-based collaborative filtering algorithm
 *
 */
public class UserBasedPredictor extends Predictor{

	private static int neighbourhoodSize;

	private static HashMap<Integer, Double> averageRatings;

	//hash map: userID --> hashmap: userID --> similarity
	HashMap<Integer, LinkedHashMap<Integer,Double>> similarities;

	//hash map: userID --> Set of movies that the user has rated
	//note that this map is sorted (==> has always the same iteration order)
	private TreeMap<Integer, HashSet<Integer>> UserRatedMovies;

	/*hashmap: userID --> hashmap: movieID --> rating
	 * for every user a hashmap of movie ratings
	 * note that this map is sorted (==> has always the same iteration order)
	 */
	private TreeMap<Integer, HashMap<Integer, Integer>> UserMovieRatings;

	//needed to sort the neighbour lists by similarity
	private static Comparator<Map.Entry<Integer, Double>> comparator;


	public UserBasedPredictor(int nSize){
		neighbourhoodSize = nSize;
		UserRatedMovies = new TreeMap<Integer, HashSet<Integer>>();
		UserMovieRatings = new TreeMap<Integer, HashMap<Integer, Integer>>();
		averageRatings = new HashMap<Integer, Double>();
		similarities = new HashMap<Integer, LinkedHashMap<Integer, Double>>();

		comparator = new Comparator<Map.Entry<Integer, Double>>() {
			public int compare( Map.Entry<Integer, Double> o1, Map.Entry<Integer, Double> o2 ) {
				return (o1.getValue()).compareTo( o2.getValue() );
			}
		};
	}

	/**
	 * This method trains the predictor using the data given in the input array 'data'
	 * @param data: an array of triples (user, movie, rating)
	 */
	@Override
	public void train(int[][] data) {
		initializeUserMovieRatings(data);
		initializeUserRatedMovies(data);

		computeUserAverageRatings(data);

		// TODO Compute all pairs similarity between users and add them to the hash map similarities
		double sim = 0.0;
		

		//TODO sort the similarities by decreasing similarity and take the N most similar users
		for (Map.Entry<Integer, LinkedHashMap<Integer, Double>> entry : similarities.entrySet()){
			ArrayList<Entry<Integer, Double>> list = new ArrayList<Entry<Integer, Double>>(entry.getValue().entrySet());

			//sort by decreasing similarity
			Collections.sort(list, Collections.reverseOrder(comparator));

			//put the first N entry of list back into the LinkedHashMap
			

		}
	}



	/**
	 * This method computes the predicted rating of user 'userID' for the movie 'movieID'
	 * using the adjusted weighted-sum prediction
	 * @param userID: the id of the user (as given in the ml100 data file)
	 * @param movieID: the id of the movie (as given in the ml100 data file)
	 * @return: predicted rating
	 */
	@Override
	public double predict(int userID, int movieID) {


		double prediction = averageRatings.get(userID); 

		// TODO compute the adjusted weighted-sum prediction: Eq. 20.5 in the lecture notes
		return prediction;
	}

	/**
	 * Computes the similarity between user1 and user2 using the Pearson similarity coefficient
	 * @param user1
	 * @param user2
	 * @return similarity
	 */
	public double computeSimilarity(int user1,int user2){

		double sim = 0;


		//TODO compute the similarity (pearson correlation coefficient): Eq. 20.2 in the lecture notes

		return sim;
	}

	/**
	 * Populate the hash map UserRatedMovies with the data from the input file 
	 * @param data: the input data file
	 */
	public void initializeUserRatedMovies(int[][] data){
		for (int i = 0; i< data.length; i++){
			//user already in the hashmap
			if (UserRatedMovies.containsKey(data[i][0])) {
				HashSet<Integer> movies = UserRatedMovies.get(data[i][0]);
				movies.add(data[i][1]);
			}
			else{
				HashSet<Integer> movies = new HashSet<Integer>();
				movies.add(data[i][1]);
				UserRatedMovies.put(data[i][0], movies);
			}

		}
	}

	/**
	 * Populate the hash set UserMovieRatings with the data from the input file
	 * @param data: the input data file
	 */
	public void initializeUserMovieRatings(int[][] data){
		for (int i = 0; i< data.length; i++){
			if (UserMovieRatings.containsKey(data[i][0])) {
				UserMovieRatings.get(data[i][0]).put(data[i][1], data[i][2]);
			}
			else{
				HashMap<Integer, Integer> ratings = new HashMap<Integer,Integer>();
				ratings.put(data[i][1], data[i][2]);
				UserMovieRatings.put(data[i][0], ratings);
			}
		}
	}

	/**
	 * Compute the average ratings for all users
	 * @param data: input data
	 */
	public void computeUserAverageRatings(int[][] data){

		for (Integer userID : UserRatedMovies.keySet()){
			double rating = 0;
			for(Integer movieID : UserRatedMovies.get(userID)){
				rating += UserMovieRatings.get(userID).get(movieID);
			}
			rating /= (double) UserRatedMovies.get(userID).size();
			averageRatings.put(userID, rating);
		}
	}

	public HashMap<Integer, LinkedHashMap<Integer, Double>> getSimilarities() {
		return similarities;
	}



}
